Package: plm
Version: 1.6-5
Date: 2016-12-02
Title: Linear Models for Panel Data
Authors@R: c(person(given = "Yves",      family = "Croissant",    role = c("aut", "cre"), email = "yves.croissant@univ-reunion.fr"),
             person(given = "Giovanni",  family = "Millo",        role = "aut", email = "Giovanni.Millo@Generali.com"),
             person(given = "Kevin",     family = "Tappe",        role = "aut", email = "kevin.tappe@bwi.uni-stuttgart.de"),
      	     person(given = "Ott",       family = "Toomet",       role = "ctb", email = "otoomet@gmail.com"),
      	     person(given = "Christian", family = "Kleiber",      role = "ctb", email = "Christian.Kleiber@unibas.ch"), 
      	     person(given = "Achim",     family = "Zeileis",      role = "ctb", email = "Achim.Zeileis@R-project.org"),
      	     person(given = "Arne",      family = "Henningsen",   role = "ctb", email = "arne.henningsen@googlemail.com"),
      	     person(given = "Liviu",     family = "Andronic",     role = "ctb", email = "landronimirc@gmail.com"),
      	     person(given = "Nina",      family = "Schoenfelder", role = "ctb", email = "nina.schoenfelder@fernuni-hagen.de"))
Depends: R (>= 2.12.0), Formula (>= 0.2-0), stats
Imports: MASS, bdsmatrix, zoo, nlme, sandwich, lattice, car, lmtest
Suggests: AER
Description: A set of estimators and tests for panel data.
License: GPL (>= 2)
URL: https://cran.r-project.org/package=plm,
        https://r-forge.r-project.org/projects/plm/
Author: Yves Croissant [aut, cre],
  Giovanni Millo [aut],
  Kevin Tappe [aut],
  Ott Toomet [ctb],
  Christian Kleiber [ctb],
  Achim Zeileis [ctb],
  Arne Henningsen [ctb],
  Liviu Andronic [ctb],
  Nina Schoenfelder [ctb]
Maintainer: Yves Croissant <yves.croissant@univ-reunion.fr>
Repository: CRAN
Repository/R-Forge/Project: plm
Repository/R-Forge/Revision: 371
Repository/R-Forge/DateTimeStamp: 2016-12-02 10:18:21
Date/Publication: 2016-12-02 23:31:22
NeedsCompilation: no
Packaged: 2016-12-02 13:10:45 UTC; rforge
Built: R 3.3.2; ; 2017-05-04 08:51:09 UTC; windows
