Package: plm
Version: 2.6-4
Date: 2024-04-01
Title: Linear Models for Panel Data
Authors@R: c(person(given = "Yves",      family = "Croissant",    role = c("aut"), email = "yves.croissant@univ-reunion.fr"),
             person(given = "Giovanni",  family = "Millo",        role = c("aut"), email = "giovanni.millo@deams.units.it"),
             person(given = "Kevin",     family = "Tappe",        role = c("aut", "cre"), email = "kevin.tappe@bwi.uni-stuttgart.de"),
             person(given = "Ott",       family = "Toomet",       role = "ctb", email = "otoomet@gmail.com"),
             person(given = "Christian", family = "Kleiber",      role = "ctb", email = "Christian.Kleiber@unibas.ch"), 
             person(given = "Achim",     family = "Zeileis",      role = "ctb", email = "Achim.Zeileis@R-project.org"),
             person(given = "Arne",      family = "Henningsen",   role = "ctb", email = "arne.henningsen@googlemail.com"),
             person(given = "Liviu",     family = "Andronic",     role = "ctb"),
             person(given = "Nina",      family = "Schoenfelder", role = "ctb"))
Depends: R (>= 3.2.0)
Imports: MASS, bdsmatrix, collapse (>= 1.8.9), zoo, nlme, sandwich,
        lattice, lmtest, maxLik, Rdpack, Formula, stats
Suggests: AER, car, statmod, urca, pder, texreg, knitr, rmarkdown,
        fixest, lfe
Description: A set of estimators for models and (robust) covariance matrices, and tests for panel data
             econometrics, including within/fixed effects, random effects, between, first-difference, 
             nested random effects as well as instrumental-variable (IV) and Hausman-Taylor-style models,
             panel generalized method of moments (GMM) and general FGLS models,
             mean groups (MG), demeaned MG, and common correlated effects (CCEMG) and pooled (CCEP) estimators
             with common factors, variable coefficients and limited dependent variables models.
             Test functions include model specification, serial correlation, cross-sectional dependence,
             panel unit root and panel Granger (non-)causality. Typical references are general econometrics 
             text books such as Baltagi (2021), Econometric Analysis of Panel Data (<doi:10.1007/978-3-030-53953-5>),
             Hsiao (2014), Analysis of Panel Data (<doi:10.1017/CBO9781139839327>), and Croissant and Millo (2018), 
             Panel Data Econometrics with R (<doi:10.1002/9781119504641>).
License: GPL (>=2)
VignetteBuilder: knitr
URL: https://cran.r-project.org/package=plm,
        https://github.com/ycroissant/plm
BugReports: https://github.com/ycroissant/plm/issues
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
RdMacros: Rdpack
Encoding: UTF-8
Author: Yves Croissant [aut],
  Giovanni Millo [aut],
  Kevin Tappe [aut, cre],
  Ott Toomet [ctb],
  Christian Kleiber [ctb],
  Achim Zeileis [ctb],
  Arne Henningsen [ctb],
  Liviu Andronic [ctb],
  Nina Schoenfelder [ctb]
Maintainer: Kevin Tappe <kevin.tappe@bwi.uni-stuttgart.de>
Built: R 4.4.0; ; 2024-04-28 19:08:45 UTC; unix
